@@ -1,4 +1,4 @@
-# Copyright (c) 2009 Six Apart Ltd.
+# Copyright (c) 2009-2010 Six Apart Ltd.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -353,7 +353,6 @@ class TestTypePad(unittest.TestCase):
         typepad.client.complete_batch()
 
         self.assertValidAsset(asset)
-        self.assert_(not asset.can_delete)
         self.assertForbidden(asset.delete)
 
     @attr(user='group')
@@ -373,7 +372,6 @@ class TestTypePad(unittest.TestCase):
         typepad.client.complete_batch()
 
         self.assertValidAsset(asset)
-        self.assert_(not asset.can_delete)
         self.assertForbidden(asset.delete)
 
     @attr(user='admin')
@@ -395,7 +393,6 @@ class TestTypePad(unittest.TestCase):
         typepad.client.complete_batch()
 
         self.assertValidAsset(asset)
-        self.assert_(asset.can_delete)
         asset.delete()
 
         # now, see if we can select it. hopefully this fails.
@@ -422,7 +419,6 @@ class TestTypePad(unittest.TestCase):
         typepad.client.complete_batch()
 
         self.assertValidAsset(asset)
-        self.assert_(asset.can_delete)
         asset.delete()
 
         # now, see if we can select it. hopefully this fails.
@@ -445,7 +441,6 @@ class TestTypePad(unittest.TestCase):
             typepad.client.complete_batch()
 
             self.assertValidAsset(asset)
-            self.assert_(asset.can_delete)
             asset.delete()
 
             # now, see if we can select it. hopefully this fails.
@@ -470,7 +465,6 @@ class TestTypePad(unittest.TestCase):
             typepad.client.complete_batch()
 
             self.assertValidAsset(asset)
-            self.assert_(asset.can_delete)
             asset.delete()
 
             # now, see if we can select it. hopefully this fails.
@@ -723,7 +717,7 @@ class TestTypePad(unittest.TestCase):
 
             self.testdata['assets_created'].append(posted_asset.xid)
         else: # test for a failure
-            self.assertEquals(int(params['status'][0]), httplib.UNAUTHORIZED)
+            self.assertEquals(int(params['status'][0]), httplib.FORBIDDEN)
 
     @attr(user='member')
     def test_5_POST_browser_upload__photo__by_member(self):
@@ -1165,7 +1159,7 @@ class TestTypePad(unittest.TestCase):
         rel = followers[0]
         self.assertValidRelationship(rel)
 
-        rel_link = rel.links['self'].href
+        rel_link = rel.make_self_link()
 
         typepad.client.batch_request()
         rel2 = typepad.Relationship.get(rel_link)
@@ -1192,11 +1186,11 @@ class TestTypePad(unittest.TestCase):
         rel = followers[0]
         self.assertValidRelationship(rel)
 
-        rel_status = rel.links['status'].href
-
         typepad.client.batch_request()
-        status = typepad.RelationshipStatus.get(rel_status)
-        typepad.client.complete_batch()
+        try:
+            status = rel.status_obj
+        finally:
+            typepad.client.complete_batch()
 
         self.assertValidRelationshipStatus(status)
 
@@ -1228,7 +1222,7 @@ class TestTypePad(unittest.TestCase):
             memberships[0].block()
         elif action == 'unblock':
             self.assert_(memberships[0].is_blocked())
-            status = typepad.RelationshipStatus.get(memberships[0].status_url(), batch=False)
+            status = typepad.RelationshipStatus.get(memberships[0].status_obj._location, batch=False)
             status.types = ["tag:api.typepad.com,2009:Member"]
             status.put()
 
@@ -1879,46 +1873,27 @@ class TestTypePad(unittest.TestCase):
         object_type = asset.primary_object_type()
         self.assert_(object_type)
         self.assert_(asset.published)
-        self.assert_(len(asset.links) > 0)
-        self.assert_('self' in asset.links)
-        self.assert_(asset.links['self'].href)
-        self.assertEquals(asset.links['self'].type, 'application/json')
-        self.assert_('favorites' in asset.links)
-        self.assert_(asset.links['favorites'].href)
-        self.assert_(asset.links['favorites'].type, 'application/json')
-        self.assert_(asset.links['favorites'].total >= 0)
-        self.assertEquals(asset.links['favorites'].total, asset.favorite_count())
-        self.assert_('replies' in asset.links)
-        self.assert_(asset.links['replies'].href)
-        self.assert_(asset.links['replies'].type, 'application/json')
-        self.assert_(asset.links['replies'].total >= 0)
-        self.assertEquals(asset.links['replies'].total, asset.comment_count())
-        if 'alternate' in asset.links:
-            self.assert_('alternate' in asset.links)
-            self.assert_(asset.links['alternate'].href)
-            self.assertEquals(asset.links['alternate'].type, 'text/html')
+
+        self.assert_(asset.make_self_link())
+        self.assert_(asset.favorite_count is not None)
+        self.assert_(asset.comment_count is not None)
+        # TODO: alternate was optional before, so maybe permalink_url might be None sometimes?
+        self.assert_(asset.permalink_url is not None)
+
         self.assert_(object_type.startswith('tag:api.typepad.com'))
         self.assertValidAssetRef(asset.asset_ref)
         if object_type == 'tag:api.typepad.com,2009:Link':
             # additional properties we expect for link assets
-            self.assert_('target' in asset.links)
-            self.assert_(asset.links['target'].href)
+            self.assert_(asset.target_url is not None)
         elif object_type == 'tag:api.typepad.com,2009:Photo':
             # additional properties we expect for photo assets
-            self.assert_('enclosure' in asset.links)
-            self.assert_(asset.links['enclosure'].href)
-            self.assert_(asset.links['enclosure'].height)
-            self.assert_(asset.links['enclosure'].width)
-            self.assert_(asset.links['enclosure'].type.startswith('image/'))
-            self.assert_('preview' in asset.links)
-            self.assert_(asset.links['preview'].href)
-            self.assert_(asset.links['preview'].height)
-            self.assert_(asset.links['preview'].width)
-            self.assert_(asset.links['preview'].type.startswith('image/'))
+            self.assert_(asset.image_link is not None)
+            self.assert_(asset.image_link.url_template is not None)
+            self.assert_(asset.image_link.width is not None)
+            self.assert_(asset.image_link.height is not None)
         elif object_type == 'tag:api.typepad.com,2009:Audio':
-            self.assert_('enclosure' in asset.links)
-            self.assert_(asset.links['enclosure'].href)
-            self.assert_(asset.links['enclosure'].type.startswith('audio/'))
+            self.assert_(asset.audio_link is not None)
+            self.assert_(asset.audio_link.url is not None)
         elif object_type == 'tag:api.typepad.com,2009:Post':
             pass
         elif object_type == 'tag:api.typepad.com,2009:Comment':
@@ -1927,21 +1902,19 @@ class TestTypePad(unittest.TestCase):
                 self.assert_(asset.in_reply_to)
                 self.assertValidAssetRef(asset.in_reply_to)
         elif object_type == 'tag:api.typepad.com,2009:Video':
-            self.assert_('enclosure' in asset.links)
-            self.assert_(asset.links['enclosure'].html)
-            self.assert_(asset.links['enclosure'].height)
-            self.assert_(asset.links['enclosure'].width)
-            # self.assertEquals(asset.links['enclosure'].type, 'text/html')
+            self.assert_(asset.video_link is not None)
+            self.assert_(asset.video_link.embed_code is not None)
+
+            preview = asset.preview_image_link
+            if preview is not None:
+                self.assert_(preview.url_template is not None)
+                self.assert_(preview.width is not None)
+                self.assert_(preview.height is not None)
         else:
             self.fail('asset has an unexpected objectType: %s' % \
                 object_type)
         if asset.source:
-            self.assert_(asset.source.links)
-            self.assert_(asset.source.original_link)
-            self.assert_('alternate' in asset.source.links)
-            self.assert_(asset.source.links['alternate'].href)
-            self.assertEquals(asset.source.links['alternate'].type,
-                'text/html')
+            self.assert_(asset.source.permalink_url)
             self.assert_(asset.source.provider)
 
             self.assert_(asset.source.provider.icon)
@@ -1962,43 +1935,15 @@ class TestTypePad(unittest.TestCase):
         # this asserts as false when the user's interests is empty
         # self.assert_(user.interests)
 
-        self.assert_(len(user.links) > 0)
-
-        self.assert_('self' in user.links)
-        self.assert_(user.links['self'].href)
-        self.assertEquals(user.links['self'].type, 'application/json')
-
-        self.assert_('elsewhere-accounts' in user.links)
-        self.assert_(user.links['elsewhere-accounts'].href)
-        self.assertEquals(user.links['elsewhere-accounts'].type,
-            'application/json')
-
-        self.assert_(user.links['follow-frame-content'].href)
-        self.assert_(user.links['follow-frame-content'].height)
-        self.assert_(user.links['follow-frame-content'].width)
-        self.assertEquals(user.links['follow-frame-content'].type,
-            'text/html')
-
-        self.assert_('alternate' in user.links)
-        self.assert_(user.links['alternate'].href)
-        self.assertEquals(user.links['alternate'].type, 'text/html')
-
-        # this link is available when a user is accessing their own
-        # user object
-        if 'membership-management-page' in user.links:
-            self.assert_(user.links['membership-management-page'].href)
-            self.assertEquals(user.links['membership-management-page'].type, 'text/html')
-
-        # this link is available when a user is accessing their own
-        # user object
-        if 'profile-edit-page' in user.links:
-            self.assert_(user.links['profile-edit-page'].href)
-            self.assertEquals(user.links['profile-edit-page'].type, 'text/html')
-
-        self.assert_('avatar' in user.links)
-        self.assert_(user.links['avatar'].href)
-        self.assert_(user.links['avatar'].height)
-        self.assert_(user.links['avatar'].width)
+        self.assert_(user.make_self_link())
+        self.assert_(user.profile_page_url is not None)
+        self.assert_(user.avatar_link is not None)
+        try:
+            self.assert_(user.avatar_link.url_template is not None)
+        except AssertionError:
+            self.assert_(user.avatar_link.url is not None)
+        self.assert_(user.avatar_link.width is not None)
+        self.assert_(user.avatar_link.height is not None)
 
     def assertValidEvent(self, event):
         """Checks given asset for properties that should be present on all assets."""
@@ -2009,26 +1954,25 @@ class TestTypePad(unittest.TestCase):
         self.assert_(event.xid)
         self.assert_(event.url_id)
         self.assert_(event.published)
-        self.assert_(len(event.links) > 0)
-        self.assert_('self' in event.links)
-        self.assert_(event.links['self'].href)
-        self.assertEquals(event.links['self'].type, 'application/json')
+        self.assert_(event.make_self_link())
         self.assert_(len(event.verbs) > 0)
         m = re.match(r'^tag:api\.typepad\.com,2009:([A-Za-z]+)$',
             event.verbs[0])
         self.assert_(m,
             "%s does not match '^tag:api\.typepad\.com,2009:[A-Za-z]+$'" \
             % event.verbs[0])
-        self.assert_(m.groups()[0] in ('AddedFavorite', 'NewAsset',
-            'JoinedGroup', 'AddedNeighbor'),
-            "Event type %s is not recognized" % m.groups()[0])
+        event_type = m.group(1)
         self.assert_(event.actor)
         self.assertValidUser(event.actor)
         # FIXME: https://intranet.sixapart.com/bugs/default.asp?87911
-        # self.assert_(event.object)
-        # self.assertValidAsset(event.object)
-        if event.object is not None:
+        if event.object is None:
+            pass
+        elif event_type in ('AddedFavorite', 'NewAsset'):
             self.assertValidAsset(event.object)
+        elif event_type in ('JoinedGroup', 'AddedNeighbor'):
+            self.assertValidUser(event.object)
+        else:
+            self.fail("Event type %s is not recognized" % event_type)
 
     def assertValidGroup(self, group):
         """Checks given asset for properties that should be present on all assets.
@@ -2039,10 +1983,7 @@ class TestTypePad(unittest.TestCase):
         self.assert_(group.id)
         self.assert_(group.url_id)
         self.assert_(group.display_name)
-        self.assert_(len(group.links) > 0)
-        self.assert_('self' in group.links)
-        self.assert_(group.links['self'].href)
-        self.assertEquals(group.links['self'].type, 'application/json')
+        self.assert_(group.make_self_link())
         self.assert_(len(group.object_types) > 0)
         self.assertEquals(group.object_types[0],
             'tag:api.typepad.com,2009:Group')
@@ -2050,31 +1991,13 @@ class TestTypePad(unittest.TestCase):
     def assertValidApplication(self, app):
         self.assert_(isinstance(app, typepad.Application),
             'object %r is not a typepad.Application' % app)
-        self.assert_(app.links)
-        links = ('oauth-request-token-endpoint', 'oauth-authorization-page',
-            'oauth-identification-page', 'signout-page',
-            'session-sync-script', 'oauth-access-token-endpoint',
-            'user-flyouts-script', 'self')
+        links = ('oauth_request_token_url', 'oauth_authorization_url',
+            'oauth_access_token_url', 'oauth_identification_url',
+            'session_sync_script_url', 'signout_url',
+            'user_flyouts_script_url')
         for link in links:
-            self.assert_(link in app.links, 'no %s link present' % link)
-            self.assert_(app.links[link].href, 'no href on %s link' % link)
-            if link.endswith('-script'):
-                self.assertEquals(app.links[link].type, 'text/javascript',
-                    'type %s is invalid for %s link' % \
-                    (app.links[link].type, link))
-            elif link.endswith('-page'):
-                self.assertEquals(app.links[link].type, 'text/html',
-                    'type %s is invalid for %s link' % \
-                    (app.links[link].type, link))
-        self.assert_(app.oauth_request_token)
-        self.assert_(app.oauth_authorization_page)
-        self.assert_(app.oauth_access_token_endpoint)
-        self.assert_(app.session_sync_script)
-        self.assert_(app.oauth_identification_page)
-        self.assert_(app.signout_page)
-        self.assert_(app.user_flyouts_script)
+            self.assert_(getattr(app, link) is not None)
         self.assert_(app.browser_upload_endpoint)
-        self.assertEquals(app.links['self'].type, 'application/json')
 
     def assertValidFavorite(self, fav):
         self.assert_(isinstance(fav, typepad.Favorite),
@@ -2103,10 +2026,8 @@ class TestTypePad(unittest.TestCase):
     def assertValidRelationship(self, rel):
         self.assert_(isinstance(rel, typepad.Relationship),
             'object %r is not a typepad.Relationship' % rel)
-        self.assert_(rel.links)
-        self.assert_('self' in rel.links)
-        self.assert_(rel.links['self'].href)
-        self.assertEquals(rel.links['self'].type, 'application/json')
+        self.assert_(rel.url_id)
+        self.assert_(rel.make_self_link())
         self.assert_(rel.status)
         # FIXME: https://intranet.sixapart.com/bugs/default.asp?87929
         # Most user-to-user relationships have created and status.types,
@@ -2157,23 +2078,19 @@ class TestTypePad(unittest.TestCase):
         (full, slice1, slice2, slice3, slice4) = listset
         self.assert_(isinstance(full, typepad.ListObject),
             'object %r is not a typepad.ListObject' % full)
-        # api result set should never exceed 50 entries
-        self.assert_(len(full.entries) <= 50)
-        # regardless of start-index/max-results parameters, the total-results
-        # parameter should be the same as the full selection
-        self.assert_(full.total_results > 1,
-            'list must have more than 1 item to test start-index query parameter')
-        if full.total_results < 50:
-            self.assertEquals(len(slice1.entries), slice1.total_results - 1)
-        # lets not require this for now
-        # self.assert_(full.total_results < 50, 'list must have fewer than 50 items to test properly')
-        self.assertEquals(len(slice2.entries), 1)
-        self.assertEquals(len(slice3.entries), 1)
-        self.assertEquals(len(slice4.entries), 0)
-        self.assertEquals(full.total_results, slice1.total_results)
-        self.assertEquals(full.total_results, slice2.total_results)
-        self.assertEquals(full.total_results, slice3.total_results)
-        self.assertEquals(full.total_results, slice4.total_results)
+
+        self.assert_(len(full.entries) <= 50, 'API result exceeded 50 entries')
+
+        self.assertEquals(len(slice2.entries), 1, 'Slice %r had %d entries (expected 1)'
+            % (slice2, len(slice2.entries)))
+        if full.total_results > 1:
+            self.assertEquals(len(slice3.entries), 1, 'Slice %r had %d entries (expected 1)'
+                % (slice3, len(slice3.entries)))
+        else:
+            self.assertEquals(len(slice3.entries), 0, 'Slice %r had %d entries (expected 0)'
+                % (slice3, len(slice3.entries)))
+        self.assertEquals(len(slice4.entries), 0, 'Slice %r had %d entries (expected 0)'
+            % (slice4, len(slice4.entries)))
 
     def assertForbidden(self, *args, **kwargs):
         self.assertRaises(HttpObject.Forbidden, *args, **kwargs)
@@ -2195,31 +2112,19 @@ class TestTypePad(unittest.TestCase):
 
     def video_asset(self):
         """Creates a Video asset instance for testing purposes."""
-
         video = typepad.Video()
-
-        rel = typepad.Link()
-        rel.href = 'http://www.youtube.com/watch?v=pWdZTqHtJ3U'
-        rel.rel = 'enclosure'
         video.title = ''
-        video.links = typepad.LinkSet()
-        video.links.add(rel)
+        video.video_link = typepad.VideoLink(
+            permalink_url='http://www.youtube.com/watch?v=pWdZTqHtJ3U')
         video.content = 'Test video post'
-
         return video
 
     def link_asset(self):
         """Creates a Link asset instance for testing purposes."""
-
         link = typepad.LinkAsset()
-        rel = typepad.Link()
-        rel.href = 'http://www.typepad.com/'
-        rel.rel = 'target'
+        link.target_url = 'http://www.typepad.com/'
         link.title = ''
-        link.links = typepad.LinkSet()
-        link.links.add(rel)
         link.content = 'Test link post'
-
         return link
 
 if __name__ == '__main__':
